functions {
  real si_model( real lambda, real age ){
    real p;
    p = 1-exp(-lambda * age );
    return p;
  }
  
  real sis_model( real lambda, real gamma, real age){
    real p;
    p = - lambda * expm1(-(gamma+lambda)*age) / (lambda+gamma);
    return p;
  }
  
  // Sum to zero constraints: 
  // https://discourse.mc-stan.org/t/test-soft-vs-hard-sum-to-zero-constrain-choosing-the-right-prior-for-soft-constrain/3884/32
  vector Q_sum_to_zero_QR(int N) {
    vector [2*N] Q_r;
  
    for(i in 1:N) {
      Q_r[i] = -sqrt((N-i)/(N-i+1.0));
      Q_r[i+N] = inv_sqrt((N-i) * (N-i+1));
    }
    return Q_r;
  }
  
  vector sum_to_zero_QR(vector x_raw, vector Q_r) {
    int N = num_elements(x_raw) + 1;
    vector [N] x;
    real x_aux = 0;
  
    for(i in 1:N-1){
      x[i] = x_aux + x_raw[i] * Q_r[i];
      x_aux = x_aux + x_raw[i] * Q_r[i+N];
    }
    x[N] = x_aux;
    return x;
  }
}

data {
  // https://github.com/stan-dev/stan/issues/2377
  int<lower=0, upper=1> use_sis_model;

  int<lower=1>       n;
  
  int<lower=1>       n_species;
  int<lower=1>       n_region;
  int<lower=1>       n_sample_type;
  int<lower=1>       n_outdoor;
  
  int<lower=1>       n_tot[n];
  int<lower=0>       n_pos[n];
  
  int<lower=1, upper=n_species>     species[n];
  int<lower=1, upper=n_region>      region[n];
  int<lower=1, upper=n_sample_type> sample_type[n];
  int<lower=1, upper=n_outdoor>     outdoor[n];

  vector<lower=0>[n] age_low;
  vector<lower=0>[n] age_high;
  vector<lower=0>[n] age;
  
  vector<lower=0>[n] weight;
}

transformed data{
  real sigma_species_raw     = inv_sqrt(1 - inv(n_species));
  real sigma_region_raw      = inv_sqrt(1 - inv(n_region));
  real sigma_sample_type_raw = inv_sqrt(1 - inv(n_sample_type));
  real sigma_outdoor_raw     = inv_sqrt(1 - inv(n_outdoor));
  
  vector[2*n_species]     Q_species     = Q_sum_to_zero_QR(n_species);
  vector[2*n_region]      Q_region      = Q_sum_to_zero_QR(n_region);
  vector[2*n_sample_type] Q_sample_type = Q_sum_to_zero_QR(n_sample_type);
  vector[2*n_outdoor]     Q_outdoor     = Q_sum_to_zero_QR(n_outdoor);
}

parameters {
  real  lambda_baseline;
  
  vector[n_region-1]      lambda_region_raw;
  vector[n_species-1]     lambda_species_raw;
  vector[n_sample_type-1] lambda_sample_type_raw;
  vector[n_outdoor-1]     lambda_outdoor_raw;

  real                    gamma_baseline[use_sis_model];
  vector[n_species-1]     gamma_species_raw[use_sis_model];
  real<lower=0>           sigma_gamma_species[use_sis_model];
  
  real<lower=0>           sigma_lambda_region;  
  real<lower=0>           sigma_lambda_species;
  real<lower=0>           sigma_lambda_sample_type;
  real<lower=0>           sigma_lambda_outdoor;
  
  vector<lower=0, upper=1>[n] age_raw;
}


transformed parameters{
  vector[n_species] gamma_species[use_sis_model];

  
  vector[n_region]      lambda_region      = sum_to_zero_QR(lambda_region_raw, Q_region );
  vector[n_species]     lambda_species     = sum_to_zero_QR(lambda_species_raw, Q_species );
  vector[n_sample_type] lambda_sample_type = sum_to_zero_QR(lambda_sample_type_raw, Q_sample_type );
  vector[n_outdoor]     lambda_outdoor     = sum_to_zero_QR(lambda_outdoor_raw, Q_outdoor );
  
  gamma_species[use_sis_model] = sum_to_zero_QR(gamma_species_raw[use_sis_model], Q_species );
}

model {
  vector[n] p;
  real      lambda[n];
  real      gamma[use_sis_model, n];
  vector[n] agedist;
 
  age_raw ~ beta_proportion( (age-age_low)./(age_high-age_low), 10 );

  agedist = age_low + (age_high - age_low) .* age_raw;
  
  // Centered parametrisation: sigma * N(0,1) instead of N(0,sigma)
  // Complicated expression for the SD below ensures sum-to-one.
  // https://mc-stan.org/docs/2_27/stan-users-guide/parameterizing-centered-vectors.html
  lambda_baseline         ~ normal( 1, 2 );
  
  lambda_region_raw       ~ normal(0, sigma_region_raw);
  lambda_species_raw      ~ normal(0, sigma_species_raw );
  lambda_sample_type_raw  ~ normal(0, sigma_sample_type_raw);
  lambda_outdoor_raw      ~ normal(0, sigma_outdoor_raw);
  
  sigma_lambda_region        ~ std_normal(  );
  sigma_lambda_species       ~ std_normal(  );
  sigma_lambda_sample_type   ~ std_normal(  );
  sigma_lambda_outdoor       ~ std_normal(  );

  if( use_sis_model ){
    sigma_gamma_species[use_sis_model] ~ std_normal();
    gamma_baseline[use_sis_model] ~ normal( 1, 2);
    gamma_species_raw[use_sis_model]~ normal(0, sigma_species_raw);
  
    for( j in 1:n ){
      gamma[use_sis_model,j] = exp( gamma_baseline[use_sis_model] + 
                                      sigma_gamma_species[use_sis_model] * 
                                      gamma_species[use_sis_model,species[j]] );
    }
  }
  
  for( j in 1:n )
    lambda[j] = exp(lambda_baseline + 
                     sigma_lambda_region      *lambda_region[region[j]] +
                     sigma_lambda_species     *lambda_species[species[j]] +
                     sigma_lambda_sample_type *lambda_sample_type[sample_type[j]] +
                     sigma_lambda_outdoor     *lambda_outdoor[outdoor[j]] );
                     
  if( use_sis_model ){    
    for( j in 1:n )
      p[j] = sis_model( lambda[j], 
                        gamma[1,j], //first index has size one
                        agedist[j] ); 
  }else{
    for( j in 1:n )
      p[j] = si_model( lambda[j], agedist[j] );
  }
  
  // Version with weighting of the observations, because some populations
  // are tested multiple times they should be downweighted.
  for( i in 1:n ){
    target += weight[i] * binomial_lpmf(n_pos[i]| n_tot[i], p[i] );
  }
}

// This bit only needed for 'loo' package, WAIC criterion
// generated quantities{
//   real log_lik[n];
//   for( i in 1:n ){
//     log_lik[i] = weight[i] * binomial_lpmf(n_pos[i]| n_tot[i], p[i] );
//  
//   }
// }
